#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import Kpi, ruleLimit

from ..graphql.input.edit_rule_limit_input import EditRuleLimitInput
from ..graphql.input.add_rule_limit_input import AddRuleLimitInput
from ..graphql.mutation.add_rule_limit import addRuleLimit
from ..graphql.mutation.edit_rule_limit import editRuleLimit
from ..graphql.mutation.remove_rule_limit import removeRuleLimit
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_rule_limit(
    client: SymphonyClient, 
            number: int,
            limitType: str,
            comparator: str,
            rule: str,
) -> ruleLimit:
    """This function adds Rule Limit.

    :param number: number
    :type number: str
    :param id: ID
    :type id: str
    :param limitType: limitType
    :type limitType: str
    :param rule: rule
    :type rule: psym.common.data_class.rule
    :param comparator: comparator
    :type comparator: psym.common.data_class.comparator

    :return: ruleLimit object
    :rtype: :class:`~psym.common.data_class.ruleLimit`


    **Example 1**

    .. code-block:: python

        domain = client.add_domain(
            name="domain",
        )
        kpi_category = client.add_kpi_category(
            name="kpi_category",
        )

        Kpi = client.add_Kpi(
            name="Kpi_2",
            description="new kpi",
            status=True,
            domain=domain.id,
            kpiCategory=kpi_category.id
        )

        threshold = client.add_Threshold(
            name="Threshold",
            description="threshold",
            status=True,
            kpi=Kpi.id
        )
        rule_type = client.add_rule_type(
            name="rule_type",
        )
        event_severity = client.add_event_severity(
            name="event_severity",
        )
        rule = client.add_rule(
            name="rule_1",
            additionalInfo="none",
            eventTypeName="none",
            gracePeriod=2,
            specificProblem="none",
            status=True,
            threshold=threshold.id,
            ruleType=rule_type.id,
            eventSeverity=event_severity.id,
            )

        comparator = client.add_comparator(
            name="comparator",
        )

        new_rule_limit = client.add_rule_limit(
            number=1,
            limitType="none",
            comparator=comparator.id,
            rule=rule.id

        )
    """
    rule_limit_input = AddRuleLimitInput(            
            number= number,
            limitType=limitType,
            comparator=comparator,
            rule=rule,)
    if (number ==int) or (len(limitType) != 0) or (len(comparator) != 0) or (len(rule) != 0) :
        result = addRuleLimit.execute(client, input=rule_limit_input)
        return ruleLimit(            
                id=result.id,
                number=result.number,
                limitType=result.limitType,
                comparator=result.comparator,
                rule=result.rule,)
    else:
        print ("one of the attributes is wrong, check the documentation.")


def edit_rule_limit(
    client: SymphonyClient,
    rule_limit: ruleLimit,
    new_number: Optional[int] = None,
    limitType: str= None,
    comparator: str= None,
    rule: str= None,
) -> None:
    """This function edits Rule Limit.

    :param rule_limit: Rule Limit entity
    :type name: str
    :param new_number: Rule Limit number
    :type name: int

    :return: none object
    :rtype: :class:`~psym.common.data_class.ruleLimit`

    **Example 1**

    .. code-block:: python

        new_rule_limit = client.edit_rule_limit(
            rule_limit=ruleLimit,
            new_number=new_number,
            limitType="none",
            comparator=comparator.id,
            rule=rule.id


        )
    """
    params: Dict[str, Any] = {}
    if new_number is not None:
        params.update({"_name_": new_number})
    if new_number is not None:
        editRuleLimit.execute(client, input=EditRuleLimitInput(
        id=rule_limit.id, 
        number= new_number,
        limitType=limitType,
        comparator=comparator,
        rule=rule))


def remove_rule_limit(client: SymphonyClient, id: str) -> None:
    """This function delete Rule Limit.

    :param name: Rule Limit name
    :type name: :class:`~psym.common.data_class.ruleLimit`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_rule_limit(id=123456798)
    """
    removeRuleLimit.execute(client, id=id)


#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config


# fmt: off
QUERY: List[str] = ["""
query SearchQuery(
  $name: String!
  $after: Cursor
  $first: Int = 10
  $before: Cursor
  $last: Int
) {
  searchForNode(
    name: $name
    after: $after
    first: $first
    before: $before
    last: $last
  ) {
    edges {
      node {
        typename: __typename
        ... on Location {
          id
          externalId
          name
          locationType {
            name
          }
        }
      }
    }
  }
}

"""
]


class SearchQuery:
    @dataclass(frozen=True)
    class SearchQueryData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class SearchNodesConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class SearchNodeEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Node(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class LocationType(DataClassJsonMixin):
                        name: str

                    typename: str
                    id: str
                    externalId: Optional[str]
                    name: str
                    locationType: LocationType

                node: Optional[Node]

            edges: List[SearchNodeEdge]

        searchForNode: SearchNodesConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, name: str, after: Optional[str] = None, first: Optional[int] = 10, before: Optional[str] = None, last: Optional[int] = None) -> SearchQueryData.SearchNodesConnection:
        variables: Dict[str, Any] = {"name": name, "after": after, "first": first, "before": before, "last": last}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.SearchQueryData.from_dict(response_text)
        return res.searchForNode

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, name: str, after: Optional[str] = None, first: Optional[int] = 10, before: Optional[str] = None, last: Optional[int] = None) -> SearchQueryData.SearchNodesConnection:
        variables: Dict[str, Any] = {"name": name, "after": after, "first": first, "before": before, "last": last}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.SearchQueryData.from_dict(response_text)
        return res.searchForNode



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import NetworkType, domain

from ..graphql.input.edit_network_type_input import EditNetworkTypeInput
from ..graphql.input.add_network_type_input import AddNetworkTypeInput
from ..graphql.mutation.add_network_type import addNetworkType
from ..graphql.mutation.edit_network_type import editNetworkType
from ..graphql.mutation.remove_network_type import removeNetworkType
from ..graphql.query.network_types import networkTypes
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_network_type(
    client: SymphonyClient, name: str
) -> NetworkType:
    """This function adds Network Type.

    :param name: Network Type name
    :type name: str

    :return: networkType object
    :rtype: :class:`~psym.common.data_class.networkType`


    **Example 1**

    .. code-block:: python

        new_network_type = client.add_network_type(
            name="network_type",
        )
        print(new_network_type)
    """
    network_type_input = AddNetworkTypeInput(name=name)
    if (len(name) != 0):
        result = addNetworkType.execute(client, input=network_type_input)
        return NetworkType(name=result.name, id=result.id)
    else:
        print ("you cant leave the name parameter empty")

def edit_network_type(
    client: SymphonyClient,
    network_type: NetworkType,
    new_name: Optional[str] = None,
) -> None:
    """This function edits Network Type.

    :param network_type: Network Type entity
    :type name: str
    :param new_name: Network Type name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.networkType`

    **Example 1**

    .. code-block:: python

        new_network_type = client.edit_network_type(
            network_type=networkType,
            new_name="network_type_edit",

        )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editNetworkType.execute(client, input=EditNetworkTypeInput(id=network_type.id, name=new_name))


def get_network_types(client: SymphonyClient) -> Iterator[NetworkType]:
    """ this funtion Get Network Types


    :return: networkType object
    :rtype: Iterator[ :class:`~psym.common.data_class.networkType` ]

    **Example**

    .. code-block:: python

        network_types = client.get_network_types()
        for network_type in network_types:
            print(network_type.name)
    """
    network_types_ = networkTypes.execute(client, first=PAGINATION_STEP)
    edges = network_types_.edges if network_types_ else []
    while network_types_ is not None and network_types_.pageInfo.hasNextPage:
        network_types_ = networkTypes.execute(
            client, after=network_types_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if network_types_ is not None:
            edges.extend(network_types_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield NetworkType(
                id=node.id,
                name=node.name,
            )


def remove_network_type(client: SymphonyClient, id: str) -> None:
    """This function delete Network Type.

    :param name: Network Type name
    :type name: :class:`~psym.common.data_class.networkType`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_network_type(id=12346789)
    """
    removeNetworkType.execute(client, id=id)


#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class SurveyQuestionType(Enum):
    BOOL = "BOOL"
    EMAIL = "EMAIL"
    COORDS = "COORDS"
    PHONE = "PHONE"
    TEXT = "TEXT"
    TEXTAREA = "TEXTAREA"
    PHOTO = "PHOTO"
    WIFI = "WIFI"
    CELLULAR = "CELLULAR"
    FLOAT = "FLOAT"
    INTEGER = "INTEGER"
    DATE = "DATE"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "SurveyQuestionType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class SurveyWiFiScanData(DataClassJsonMixin):
    timestamp: int
    frequency: int
    channel: int
    bssid: str
    strength: int
    ssid: Optional[str] = None
    band: Optional[str] = None
    channelWidth: Optional[int] = None
    capabilities: Optional[str] = None
    latitude: Optional[Number] = None
    longitude: Optional[Number] = None
    altitude: Optional[Number] = None
    heading: Optional[Number] = None
    rssi: Optional[Number] = None



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Dict, Iterator, Mapping, Optional, Tuple, cast

from tqdm import tqdm

from psym.client import SymphonyClient
from psym.common.cache import EQUIPMENT_TYPES
from psym.common.constant import EQUIPMENTS_TO_SEARCH, PAGINATION_STEP
from psym.common.data_class import Equipment, EquipmentType, Location
from psym.common.data_enum import Entity
from psym.common.data_format import format_to_equipment

from .._utils import (
    PropertyValue,
    _get_property_value,
    get_graphql_property_inputs,
    update_property_input_ids,
)
from ..exceptions import (
    EntityNotFoundError,
    EquipmentIsNotUniqueException,
    EquipmentNotFoundException,
    EquipmentPositionIsNotUniqueException,
    EquipmentPositionNotFoundException,
)
from ..graphql.enum.equipment_filter_type import EquipmentFilterType
from ..graphql.enum.filter_operator import FilterOperator
from ..graphql.enum.property_kind import PropertyKind
from ..graphql.input.add_equipment_input import AddEquipmentInput
from ..graphql.input.edit_equipment_input import EditEquipmentInput
from ..graphql.input.equipment_filter_input import EquipmentFilterInput
from ..graphql.mutation.add_equipment import AddEquipmentMutation
from ..graphql.mutation.edit_equipment import EditEquipmentMutation
from ..graphql.mutation.remove_equipment import RemoveEquipmentMutation
from ..graphql.query.equipment_positions import EquipmentPositionsQuery
from ..graphql.query.equipment_search import EquipmentSearchQuery
from ..graphql.query.equipment_type_and_properties import (
    EquipmentTypeAndPropertiesQuery,
)
from ..graphql.query.equipment_type_equipments import EquipmentTypeEquipmentQuery
from ..graphql.query.equipments import EquipmentsQuery
from ..graphql.query.location_equipments import LocationEquipmentsQuery


def _get_equipment_if_exists(
    client: SymphonyClient, name: str, location: Location
) -> Optional[Equipment]:

    location_with_equipments = LocationEquipmentsQuery.execute(client, id=location.id)
    if location_with_equipments is None:
        raise EntityNotFoundError(entity=Entity.Location, entity_id=location.id)
    equipments = [
        equipment
        for equipment in location_with_equipments.equipments
        if equipment.name == name
    ]
    if len(equipments) > 1:
        raise EquipmentIsNotUniqueException(name)

    if len(equipments) == 0:
        return None
    return format_to_equipment(equipment_fragment=equipments[0])


def get_equipment(client: SymphonyClient, name: str, location: Location) -> Equipment:
    """Get equipment by name in a given location.

    :param name: Equipment name
    :type name: str
    :param location: Location object could be retrieved from

        * :meth:`~psym.api.location.get_location`
        * :meth:`~psym.api.location.add_location`

    :type location: :class:`~psym.common.data_class.Location`


    :raises:
        * EquipmentIsNotUniqueException: Location contains more than one equipment with the same name
        * EquipmentNotFoundException: The equipment was not found
        * FailedOperationException: Internal symphony error

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location([("Country", "LS_IND_Prod_Copy")])
        equipment = client.get_equipment("indProdCpy1_AIO", location)
    """

    equipment = _get_equipment_if_exists(client, name, location)
    if equipment is None:
        raise EquipmentNotFoundException(equipment_name=name)
    return equipment


def get_equipments(client: SymphonyClient) -> Iterator[Equipment]:
    """This function returns all existing equipments

    :return: Equipments Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Equipment` ]

    **Example**

    .. code-block:: python

        all_equipments = client.get_equipments()
    """
    equipments = EquipmentsQuery.execute(client, first=PAGINATION_STEP)
    edges = equipments.edges if equipments else []
    while equipments is not None and equipments.pageInfo.hasNextPage:
        equipments = EquipmentsQuery.execute(
            client, after=equipments.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if equipments is not None:
            edges.extend(equipments.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield format_to_equipment(equipment_fragment=node)


def get_equipment_by_external_id(client: SymphonyClient, external_id: str) -> Equipment:
    """Get equipment by external ID.

    :param external_id: Equipment external ID
    :type external_id: str

    :raises:
        * EquipmentIsNotUniqueException: Location contains more than one equipment with the same external ID
        * :class:`~psym.exceptions.EntityNotFoundError`: The equipment was not found
        * FailedOperationException: Internal symphony error

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        equipment = client.get_equipment_by_external_id(external_id="123456")
    """
    equipment_filter = EquipmentFilterInput(
        filterType=EquipmentFilterType.EQUIP_INST_EXTERNAL_ID,
        operator=FilterOperator.IS,
        stringValue=external_id,
        idSet=[],
        stringSet=[],
    )

    res = EquipmentSearchQuery.execute(client, filters=[equipment_filter], limit=5)

    if not res or res.totalCount == 0:
        raise EntityNotFoundError(
            entity=Entity.Equipment, msg=f"external_id={external_id}"
        )

    if res.totalCount > 1:
        raise EquipmentIsNotUniqueException(external_id)

    for edge in res.edges:
        node = edge.node
        if node is not None:
            return format_to_equipment(equipment_fragment=node)
    raise EntityNotFoundError(entity=Entity.Equipment, msg=f"external_id={external_id}")


def get_equipment_properties(
    client: SymphonyClient, equipment: Equipment
) -> Dict[str, PropertyValue]:
    """Get specific equipment properties.

    :param equipment: Equipment objecte, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`

    :return: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :rtype: Dict[str, PropertyValue]

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_equipment("indProdCpy1_AIO", location)
        properties = client.get_equipment_properties(equipment=equipment)
    """
    equipment_type, properties_dict = _get_equipment_type_and_properties_dict(
        client, equipment
    )
    return properties_dict


def get_equipments_by_type(
    client: SymphonyClient, equipment_type_id: str
) -> Iterator[Equipment]:
    """Get equipments by ID of specific type.

    :param equipment_type_id: Equipment type ID
    :type equipment_type_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Equipment type with this ID does not exist

    :return: Equipments Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Equipment` ]

    **Example**

    .. code-block:: python

        equipments = client.get_equipments_by_type(equipment_type_id="34359738369")
    """
    equipment_type_with_equipments = EquipmentTypeEquipmentQuery.execute(
        client, id=equipment_type_id
    )
    if not equipment_type_with_equipments:
        raise EntityNotFoundError(
            entity=Entity.EquipmentType, entity_id=equipment_type_id
        )
    for equipment in equipment_type_with_equipments.equipments:
        yield format_to_equipment(equipment_fragment=equipment)


def get_equipments_by_location(
    client: SymphonyClient, location_id: str
) -> Iterator[Equipment]:
    """Get equipments by ID of specific location.

    :param location_id: Location ID
    :type location_id: str

    :raises:
        :class:`~psym.exceptions.EntityNotFoundError`: Location with this ID does not exist

    :return: Equipments Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.Equipment` ]

    **Example**

    .. code-block:: python

        equipments = client.get_equipments_by_location(location_id="60129542651")
    """
    location_details = LocationEquipmentsQuery.execute(client, id=location_id)
    if location_details is None:
        raise EntityNotFoundError(entity=Entity.Location, entity_id=location_id)

    for equipment in location_details.equipments:
        yield format_to_equipment(equipment_fragment=equipment)


def _get_equipment_in_position_if_exists(
    client: SymphonyClient, parent_equipment: Equipment, position_name: str
) -> Optional[Equipment]:
    _, equipment = _find_position_definition_id(client, parent_equipment, position_name)
    return equipment


def get_equipment_in_position(
    client: SymphonyClient, parent_equipment: Equipment, position_name: str
) -> Equipment:
    """Get the equipment attached in a given `position_name` of a given `parent_equipment`

    :param parent_equipment: Parent equipment, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type parent_equipment: :class:`~psym.common.data_class.Equipment`
    :param position_name: Position name
    :type position_name: str

    :raises:
        * AssertionException: Parent equipment has more than one position with the given name,
          or none with this name or the position is not occupied.
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: `parent_equipment` does not exist

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location([("Country", "LS_IND_Prod_Copy")])
        p_equipment = client.get_equipment("indProdCpy1_AIO", location)
        equipment = client.get_equipment_in_position(p_equipment, "some_position")
    """

    equipment = _get_equipment_in_position_if_exists(
        client=client, parent_equipment=parent_equipment, position_name=position_name
    )
    if equipment is None:
        raise EquipmentNotFoundException(
            parent_equipment_name=parent_equipment.name,
            parent_position_name=position_name,
        )
    return equipment


def add_equipment(
    client: SymphonyClient,
    name: str,
    equipment_type: str,
    location: Location,
    properties_dict: Mapping[str, PropertyValue],
    external_id: Optional[str] = None,
) -> Equipment:
    """Create a new equipment in a given `location`.
    The equipment will be of the given `equipment_type`,
    with the given `name` and with the given `properties`.
    If equipment with this name already exists in this location,
    then existing equipment is returned.

    :param name: New equipment name
    :type name: str
    :param equipment_type: Equipment type name
    :type equipment_type: str
    :param location: Location object, could be retrieved from

        * :meth:`~psym.api.location.get_location`
        * :meth:`~psym.api.location.add_location`

    :type location: :class:`~psym.common.data_class.Location`
    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Mapping[str, PropertyValue]
    :param external_id: Equipment external ID
    :type external_id: str, optional

    :raises:
        * AssertionException: Location contains more than one equipment with the
          same name or property value in `properties_dict` does not match the property type
        * FailedOperationException: Internal symphony error

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        from datetime import date
        equipment = client.add_equipment(
            name="Router X123",
            equipment_type="Router",
            location=location,
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            })
    """

    property_types = EQUIPMENT_TYPES[equipment_type].property_types
    properties = get_graphql_property_inputs(property_types, properties_dict)

    add_equipment_input = AddEquipmentInput(
        name=name,
        type=EQUIPMENT_TYPES[equipment_type].id,
        location=location.id,
        properties=properties,
        externalId=external_id,
    )
    equipment = AddEquipmentMutation.execute(client, add_equipment_input)

    return format_to_equipment(equipment_fragment=equipment)


def edit_equipment(
    client: SymphonyClient,
    equipment: Equipment,
    new_name: Optional[str] = None,
    new_properties: Optional[Dict[str, PropertyValue]] = None,
) -> Equipment:
    """Edit existing equipment.

    :param equipment: Equipment object, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param new_name: Equipment new name
    :type new_name: str, optional
    :param new_properties: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type new_properties: Dict[str, PropertyValue], optional

    :raises:
        FailedOperationException: Internal symphony error

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        edited_equipment = client.edit_equipment(
            equipment=equipment,
            new_name="new_name",
            new_properties={"Z AIO - Number": 123},
        )
    """
    properties = []
    if new_properties:
        property_types = EQUIPMENT_TYPES[equipment.equipment_type_name].property_types
        existing_properties = {p.propertyType.id: p for p in equipment.properties}
        property_inputs = get_graphql_property_inputs(property_types, new_properties)
        properties = update_property_input_ids(existing_properties, property_inputs)

    edit_equipment_input = EditEquipmentInput(
        id=equipment.id,
        name=new_name if new_name else equipment.name,
        properties=properties,
    )
    result = EditEquipmentMutation.execute(client, edit_equipment_input)

    return format_to_equipment(equipment_fragment=result)


def _find_position_definition_id(
    client: SymphonyClient, equipment: Equipment, position_name: str
) -> Tuple[str, Optional[Equipment]]:

    equipment_data = EquipmentPositionsQuery.execute(client, id=equipment.id)

    if not equipment_data:
        raise EntityNotFoundError(entity=Entity.Equipment, entity_id=equipment.id)

    positions = equipment_data.equipmentType.positionDefinitions
    existing_positions = equipment_data.positions

    positions = [position for position in positions if position.name == position_name]
    if len(positions) > 1:
        raise EquipmentPositionIsNotUniqueException(equipment.name, position_name)
    if len(positions) == 0:
        raise EquipmentPositionNotFoundException(equipment.name, position_name)
    position = positions[0]

    installed_positions = [
        existing_position
        for existing_position in existing_positions
        if existing_position.definition.name == position_name
    ]
    if len(installed_positions) > 1:
        raise EquipmentIsNotUniqueException(
            parent_equipment_name=equipment.name, parent_position_name=position_name
        )
    if len(installed_positions) == 1:
        attached_equipment = installed_positions[0].attachedEquipment
        if attached_equipment is not None:
            return (
                position.id,
                format_to_equipment(equipment_fragment=attached_equipment),
            )
    return position.id, None


def add_equipment_to_position(
    client: SymphonyClient,
    name: str,
    equipment_type: str,
    existing_equipment: Equipment,
    position_name: str,
    properties_dict: Mapping[str, PropertyValue],
    external_id: Optional[str] = None,
) -> Equipment:
    """Create a new equipment inside a given `position_name` of the given `existing_equipment`.
    The equipment will be of the given `equipment_type`, with the given `name` and with the given `properties`.
    If equipment with this name already exists in this position, then existing equipment is returned.

    :param name: New equipment name
    :type name: str
    :param equipment_type: Equipment type name
    :type equipment_type: str
    :param existing_equipment: Equipment object, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type existing_equipment: :class:`~psym.common.data_class.Equipment`
    :param position_name: Position name in the equipment type
    :type position_name: str
    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Mapping[str, PropertyValue]
    :param external_id: Equipment external ID
    :type external_id: str, optional

    :raises:
        * AssertionException: Parent equipment has more than one position with the given name
          or property value in `properties_dict` does not match the property type
        * FailedOperationException: Internal symphony error
        * :class:`~psym.exceptions.EntityNotFoundError`: `existing_equipment` does not exist

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        from datetime import date
        equipment = client.add_equipment_to_position(
            name="Card Y123",
            equipment_type="Card",
            existing_equipment=equipment,
            position_name="Pos 1",
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            }
        )
    """

    position_definition_id, _ = _find_position_definition_id(
        client, existing_equipment, position_name
    )
    property_types = EQUIPMENT_TYPES[equipment_type].property_types
    properties = get_graphql_property_inputs(property_types, properties_dict)

    add_equipment_input = AddEquipmentInput(
        name=name,
        type=EQUIPMENT_TYPES[equipment_type].id,
        parent=existing_equipment.id,
        positionDefinition=position_definition_id,
        properties=properties,
        externalId=external_id,
    )
    equipment = AddEquipmentMutation.execute(client, add_equipment_input)

    return format_to_equipment(equipment_fragment=equipment)


def delete_equipment(client: SymphonyClient, equipment: Equipment) -> None:
    """This function delete Equipment.

    :param equipment: Existing equipment object, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`

    :rtype: None

    **Example**

    .. code-block:: python

        client.delete_equipment(equipment=equipment)
    """
    RemoveEquipmentMutation.execute(client, id=equipment.id)


def search_for_equipments(
    client: SymphonyClient, limit: int
) -> Tuple[Iterator[Equipment], int]:
    """Search for equipments.

    :param limit: Search result limit
    :type limit: int

    :return: Tuple[Iterator[ :class:`~psym.common.data_class.Equipment` ], int]

        * Iterator[ :class:`~psym.common.data_class.Equipment` ] - Equipments Iterator
        * int - Total count of results

    :rtype: Tuple[Iterator[ :class:`~psym.common.data_class.Equipment` ], int]

    **Example**

    .. code-block:: python

        client.search_for_equipments(limit=10)
    """

    def generate_equipments(
        data: EquipmentSearchQuery.EquipmentSearchQueryData.EquipmentConnection,
    ) -> Iterator[Equipment]:
        for edge in data.edges:
            node = edge.node
            if node is not None:
                yield format_to_equipment(equipment_fragment=node)

    equipments_result = EquipmentSearchQuery.execute(client, filters=[], limit=limit)
    total_count = equipments_result.totalCount

    equipments = generate_equipments(equipments_result)
    return equipments, total_count


def delete_all_equipments(client: SymphonyClient) -> None:
    """This function delete all Equipments.

    :rtype: None

    **Example**

    .. code-block:: python

        client.delete_all_equipment()
    """

    def delete_equipments(client: SymphonyClient) -> Tuple[int, int]:
        equipments, total = search_for_equipments(
            client=client, limit=EQUIPMENTS_TO_SEARCH
        )
        deleted = 0
        for equipment in equipments:
            deleted += 1
            delete_equipment(client=client, equipment=equipment)

        return total, deleted

    total_count, deleted_count = delete_equipments(client)
    if total_count == deleted_count:
        return

    with tqdm(total=total_count) as progress_bar:
        progress_bar.update(deleted_count)
        while deleted_count != 0:
            total_count, deleted_count = delete_equipments(client)
            progress_bar.update(deleted_count)


def _get_equipment_type_and_properties_dict(
    client: SymphonyClient, equipment: Equipment
) -> Tuple[str, Dict[str, PropertyValue]]:

    result = EquipmentTypeAndPropertiesQuery.execute(client, id=equipment.id)
    if result is None:
        raise EntityNotFoundError(entity=Entity.Equipment, entity_id=equipment.id)
    equipment_type = result.equipmentType.name

    properties_dict: Dict[str, PropertyValue] = {}
    property_types = EQUIPMENT_TYPES[equipment_type].property_types
    for property in result.properties:
        property_type_id = property.propertyType.id
        property_types_with_id = [
            property_type
            for property_type in property_types
            if property_type.id == property_type_id
        ]
        assert (
            len(property_types_with_id) == 1
        ), "Equipment type {} has two property types with same id {}".format(
            equipment_type, property_type_id
        )
        property_type = property_types_with_id[0]
        property_value = _get_property_value(
            property_type=property_type.property_kind.value, property=property
        )
        if property_type.property_kind == PropertyKind.gps_location:
            properties_dict[property_type.property_name] = (
                cast(float, property_value[0]),
                cast(float, property_value[1]),
            )
        else:
            properties_dict[property_type.property_name] = property_value[0]
    return equipment_type, properties_dict


def copy_equipment_in_position(
    client: SymphonyClient,
    equipment: Equipment,
    dest_parent_equipment: Equipment,
    dest_position_name: str,
    new_external_id: Optional[str] = None,
) -> Equipment:
    """Copy equipment in position.

    :param equipment: Equipment object to be copied, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param dest_parent_equipment: Parent equipment, destination to copy to
    :type dest_parent_equipment: :class:`~psym.common.data_class.Equipment`
    :param dest_position_name: Destination position name
    :type dest_position_name: str
    :param new_external_id: New external ID for equipment
    :type new_external_id: str, optional

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment_to_copy = client.get_equipment(name="indProdCpy1_AIO", location=location)
        parent_equipment = client.get_equipment(name="parent", location=location)
        copied_equipment = client.copy_equipment_in_position(
            equipment=equipment,
            dest_parent_equipment=parent_equipment,
            dest_position_name="destination position name",
        )
    """
    equipment_type, properties_dict = _get_equipment_type_and_properties_dict(
        client, equipment
    )
    return add_equipment_to_position(
        client=client,
        name=equipment.name,
        equipment_type=equipment_type,
        existing_equipment=dest_parent_equipment,
        position_name=dest_position_name,
        properties_dict=properties_dict,
        external_id=new_external_id,
    )


def copy_equipment(
    client: SymphonyClient,
    equipment: Equipment,
    dest_location: Location,
    new_external_id: Optional[str] = None,
) -> Equipment:
    """Copy equipment.

    :param equipment: Equipment object to be copied, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param dest_location: Destination location to copy to
    :type dest_location: :class:`~psym.common.data_class.Location`
    :param new_external_id: External ID for new equipment
    :type new_external_id: str, optional

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        new_location = client.get_location({("Country", "LS_IND_Prod")})
        copied_equipment = client.copy_equipment(
            equipment=equipment,
            dest_location=new_location,
        )
    """
    equipment_type, properties_dict = _get_equipment_type_and_properties_dict(
        client=client, equipment=equipment
    )
    return add_equipment(
        client=client,
        name=equipment.name,
        equipment_type=equipment_type,
        location=dest_location,
        properties_dict=properties_dict,
        external_id=new_external_id,
    )


def get_equipment_type_of_equipment(
    client: SymphonyClient, equipment: Equipment
) -> EquipmentType:
    """This function returns equipment type object of equipment.

    :param equipment: Equipment object to be copied, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`

    :return: EquipmentType object
    :rtype: :class:`~psym.common.data_class.EquipmentType`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        equipment_type = client.get_equipment_type_of_equipment(equipment=equipment)
    """
    equipment_type, _ = _get_equipment_type_and_properties_dict(
        client=client, equipment=equipment
    )
    return EQUIPMENT_TYPES[equipment_type]


def get_or_create_equipment(
    client: SymphonyClient,
    name: str,
    equipment_type: str,
    location: Location,
    properties_dict: Mapping[str, PropertyValue],
    external_id: Optional[str] = None,
) -> Equipment:
    """This function checks equipment existence by name in specific location,
    in case it is not found by name, creates one.

    :param name: Equipment name
    :type name: str
    :param equipment_type: Equipment type name
    :type equipment_type: str
    :param location: Location object, could be retrieved from

        * :meth:`~psym.api.location.get_location`
        * :meth:`~psym.api.location.add_location`

    :type location: :class:`~psym.common.data_class.Location`
    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Mapping[str, PropertyValue]
    :param external_id: Equipment external ID
    :type external_id: str, optional

    :raises:
        * AssertionException: Location contains more than one equipment with the
          same name or property value in `properties_dict` does not match the property type
        * FailedOperationException: Internal symphony error

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        equipment = client.get_or_create_equipment(
            name="indProdCpy1_AIO",
            equipment_type="router",
            location=location,
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            }
        )
    """
    equipment = _get_equipment_if_exists(client, name, location)
    if equipment is not None:
        return equipment
    return add_equipment(
        client=client,
        name=name,
        equipment_type=equipment_type,
        location=location,
        properties_dict=properties_dict,
        external_id=external_id,
    )


def get_or_create_equipment_in_position(
    client: SymphonyClient,
    name: str,
    equipment_type: str,
    existing_equipment: Equipment,
    position_name: str,
    properties_dict: Mapping[str, PropertyValue],
    external_id: Optional[str] = None,
) -> Equipment:
    """This function checks equipment existence by name in specific location,
    in case it is not found by name, creates one.

    :param name: Equipment name
    :type name: str
    :param equipment_type: Equipment type name
    :type equipment_type: str
    :param existing_equipment: Equipment object to be copied, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type existing_equipment: :class:`~psym.common.data_class.Equipment`
    :param position_name: Position name
    :type position_name: str
    :param properties_dict: Dictionary of property name to property value

        * str - property name
        * PropertyValue - new value of the same type for this property

    :type properties_dict: Mapping[str, PropertyValue]
    :param external_id: Equipment external ID
    :type external_id: str, optional

    :raises:
        * AssertionException: Location contains more than one equipment with the
          same name or property value in `properties_dict` does not match the property type
        * FailedOperationException: Internal symphony error

    :return: Equipment object, you can use the ID to access the equipment from the UI:
        https://{}.thesymphony.cloud/inventory/inventory?equipment={}
    :rtype: :class:`~psym.common.data_class.Equipment`

    **Example**

    .. code-block:: python

        location = client.get_location({("Country", "LS_IND_Prod_Copy")})
        e_equipment = client.get_equipment(name="indProdCpy1_AIO", location=location)
        equipment_in_position = client.get_or_create_equipment_in_position(
            name="indProdCpy1_AIO",
            equipment_type="router",
            existing_equipment=e_equipment,
            position_name="some_position",
            properties_dict={
                "Date Property": date.today(),
                "Lat/Lng Property": (-1.23,9.232),
                "E-mail Property": "user@fb.com",
                "Number Property": 11,
                "String Property": "aa",
                "Float Property": 1.23
            }
        )
    """
    equipment = _get_equipment_in_position_if_exists(
        client=client, parent_equipment=existing_equipment, position_name=position_name
    )
    if equipment is not None:
        return equipment

    return add_equipment_to_position(
        client=client,
        name=name,
        equipment_type=equipment_type,
        existing_equipment=existing_equipment,
        position_name=position_name,
        properties_dict=properties_dict,
        external_id=external_id,
    )



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Dict, List, Tuple

from psym.client import SymphonyClient
from psym.common.data_class import Equipment, Location
from psym.common.data_enum import Entity
from psym.common.data_format import format_to_equipment

from ..exceptions import EntityNotFoundError
from ..graphql.query.equipment_positions import EquipmentPositionsQuery
from ..graphql.query.location_equipments import LocationEquipmentsQuery
from .equipment import copy_equipment, copy_equipment_in_position
from .link import add_link, get_all_links_and_port_names_of_equipment


def _get_one_level_attachments_of_equipment(
    client: SymphonyClient, equipment: Equipment
) -> List[Tuple[str, Equipment]]:
    equipment_with_positions = EquipmentPositionsQuery.execute(client, id=equipment.id)
    if not equipment_with_positions:
        raise EntityNotFoundError(entity=Entity.Equipment, entity_id=equipment.id)
    attachments = []
    for position in equipment_with_positions.positions:
        attached_equipment = position.attachedEquipment
        if attached_equipment is not None:
            attachments.append(
                (
                    position.definition.id,
                    format_to_equipment(equipment_fragment=attached_equipment),
                )
            )

    return attachments


def copy_equipment_with_all_attachments(
    client: SymphonyClient, equipment: Equipment, dest_location: Location
) -> Dict[Equipment, Equipment]:
    """Copy the equipment to the new location with all its attachments

    :param equipment: Equipment object to be copied, could be retrieved from

        * :meth:`~psym.api.equipment.get_equipment`
        * :meth:`~psym.api.equipment.get_equipment_in_position`
        * :meth:`~psym.api.equipment.add_equipment`
        * :meth:`~psym.api.equipment.add_equipment_to_position`

    :type equipment: :class:`~psym.common.data_class.Equipment`
    :param dest_location: Location to copy equipment to, could be retrieved from

        * :meth:`~psym.api.location.get_location`
        * :meth:`~psym.api.location.add_location`

    :type dest_location: :class:`~psym.common.data_class.Location`

    :raises:
        FailedOperationException: Internal symphony error

    :return: Dictionary of source equipment to new equipment,
        includes the equipment given as parameter and also all the equipments attached to it

        * source equipment - :class:`~psym.common.data_class.Equipment`
        * new equipment - :class:`~psym.common.data_class.Equipment`

    :rtype: Iterator[ Tuple[ :class:`~psym.common.data_class.Equipment`, str ] ]
    """

    result = {}

    new_equipment = copy_equipment(client, equipment, dest_location)
    equipments = [(equipment, new_equipment)]

    while len(equipments) != 0:
        old_equipment, new_equipment = equipments.pop()
        result[old_equipment] = new_equipment
        attachments = _get_one_level_attachments_of_equipment(client, old_equipment)
        for position_name, child_equipment in attachments:
            new_child_equipment = copy_equipment_in_position(
                client, child_equipment, new_equipment, position_name
            )
            equipments.append((child_equipment, new_child_equipment))
    return result


def apply_location_template_to_location(
    client: SymphonyClient, template_location: Location, location: Location
) -> None:

    location_with_equipments = LocationEquipmentsQuery.execute(
        client, id=template_location.id
    )
    if not location_with_equipments:
        raise EntityNotFoundError(
            entity=Entity.Location, entity_id=template_location.id
        )
    equipments = [
        format_to_equipment(equipment_fragment=equipment)
        for equipment in location_with_equipments.equipments
    ]
    equipments_to_new_equipments = {}
    for equipment in equipments:
        # return back all and gather link ids
        equipments_to_new_equipments.update(
            copy_equipment_with_all_attachments(client, equipment, location)
        )

    link_to_equipment_and_port = {}
    connected_links = []

    for equipment in equipments_to_new_equipments.keys():
        links_and_ports = get_all_links_and_port_names_of_equipment(client, equipment)
        for link, port_name in links_and_ports:
            if link not in link_to_equipment_and_port:
                link_to_equipment_and_port[link] = (port_name, equipment)
            else:
                other_port_name, other_equipment = link_to_equipment_and_port.pop(link)
                connected_links.append(
                    (equipment, port_name, other_equipment, other_port_name)
                )

    assert (
        len(link_to_equipment_and_port) == 0
    ), "Some equipments in location are connected to equipments outside the location"

    for equipment, port_name, other_equipment, other_port_name in connected_links:
        new_equipment = equipments_to_new_equipments[equipment]
        new_other_equipment = equipments_to_new_equipments[other_equipment]
        add_link(client, new_equipment, port_name, new_other_equipment, other_port_name)



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from datetime import date, datetime, time
from numbers import Number
from typing import (
    Any,
    Dict,
    List,
    NamedTuple,
    Optional,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from psym.graphql.enum.project_priority import ProjectPriority
from psym.graphql.enum.property_kind import PropertyKind
from psym.graphql.enum.user_role import UserRole
from psym.graphql.enum.user_status import UserStatus
from psym.graphql.enum.flow_instance_status import FlowInstanceStatus
from psym.graphql.enum.work_order_priority import WorkOrderPriority
from psym.graphql.enum.work_order_status import WorkOrderStatus
from psym.graphql.fragment import document_category
from psym.graphql.fragment.equipment_port_definition import (
    EquipmentPortDefinitionFragment,
)
from psym.graphql.fragment.equipment_position_definition import (
    EquipmentPositionDefinitionFragment,
)
from psym.graphql.fragment.property import PropertyFragment

from ..graphql.enum.image_entity import ImageEntity

ReturnType = TypeVar("ReturnType")
PropertyValue = Union[date, datetime, float, int, str, bool, Tuple[float, float]]
PropertyValueType = Union[
    Type[date],
    Type[datetime],
    Type[float],
    Type[int],
    Type[str],
    Type[bool],
    Type[Tuple[float, float]],
]


class PropertyDefinition(NamedTuple):
    """
    :param property_name: Type name
    :type property_name: str
    :param property_kind: Property kind
    :type property_kind: class:`~psym.graphql.enum.property_kind.PropertyKind`
    :param default_raw_value: Default property value as a string

        * string - "string"
        * int - "123"
        * bool - "true" / "True" / "TRUE"
        * float - "0.123456"
        * date - "24/10/2020"
        * range - "0.123456 - 0.2345" / "1 - 2"
        * email - "email@some.domain"
        * gps_location - "0.1234, 0.2345"

    :type default_raw_value: str, optional
    :param id: ID
    :type id: str, optional
    :param is_fixed: Fixed value flag
    :type is_fixed: bool, optional
    :param external_id: Property type external ID
    :type external_id: str, optional
    :param is_mandatory: Mandatory value flag
    :type is_mandatory: bool, optional
    :param is_deleted: Is delete flag
    :type is_deleted: bool, optional
    """

    property_name: str
    property_kind: PropertyKind
    default_raw_value: Optional[str]
    id: Optional[str] = None
    is_fixed: Optional[bool] = False
    external_id: Optional[str] = None
    is_mandatory: Optional[bool] = False
    is_deleted: Optional[bool] = False


class DataTypeName(NamedTuple):
    """
    :param data_type: Data type
    :type data_type: :attr:`~psym.graphql.data_class.PropertyValueType`
    :param graphql_field_name: GraphQL field name, in case of `gps_location` it is Tuple[`latitudeValue`, `longitudeValue`]
    :type graphql_field_name: Tuple[str, ...]
    """

    data_type: PropertyValueType
    graphql_field_name: Tuple[str, ...]


TYPE_AND_FIELD_NAME = {
    "date": DataTypeName(data_type=date, graphql_field_name=("stringValue",)),
    "datetime_local": DataTypeName(
        data_type=datetime, graphql_field_name=("stringValue",)
    ),
    "float": DataTypeName(data_type=float, graphql_field_name=("floatValue",)),
    "int": DataTypeName(data_type=int, graphql_field_name=("intValue",)),
    "email": DataTypeName(data_type=str, graphql_field_name=("stringValue",)),
    "string": DataTypeName(data_type=str, graphql_field_name=("stringValue",)),
    "bool": DataTypeName(data_type=bool, graphql_field_name=("booleanValue",)),
    "gps_location": DataTypeName(
        data_type=tuple,  # type: ignore
        graphql_field_name=("latitudeValue", "longitudeValue"),
    ),
    "enum": DataTypeName(data_type=str, graphql_field_name=("stringValue",)),
}


class DocumentCategory(NamedTuple):
    """
    :param name: Category name
    :type name: str
    :param dc_index: Category name index
    :type dc_index: int, optional
    :param id: ID
    :type id: str, optional
    """

    name: str
    index: Optional[int] = None
    id: Optional[str] = None


class LocationType(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param property_types: PropertyTypes sequence
    :type property_types: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param map_type: Map type
    :type map_type: str, optional
    :param map_zoom_level: Map zoom level
    :type map_zoom_level: int, optional
    :param is_site: Is site flag for location
    :type is_site: bool
    """

    name: str
    id: str
    property_types: Sequence[PropertyDefinition]
    document_categories: Sequence[DocumentCategory]
    map_type: Optional[str]
    map_zoom_level: Optional[int]
    is_site: bool


class Location(NamedTuple):
    """
    :param name: name
    :type name: str
    :param id: ID
    :type id: str
    :param latitude: latitude
    :type latitude: Number
    :param longitude: longitude
    :type longitude: Number
    :param external_id: external ID
    :type external_id: str, optional
    :param location_type_name: Location type name
    :type location_type_name: str
    :param properties: PropertyFragment sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property.PropertyFragment` ])
    """

    name: str
    id: str
    latitude: Number
    longitude: Number
    external_id: Optional[str]
    location_type_name: str
    properties: Sequence[PropertyFragment]


class EquipmentType(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param category: Category
    :type category: str, optional
    :param id: ID
    :type id: str
    :param property_types: PropertyDefinitions sequence
    :type property_types: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param position_definitions: EquipmentPositionDefinitionFragments sequence
    :type position_definitions: Sequence[ :class:`~psym.graphql.fragment.equipment_position_definition.EquipmentPositionDefinitionFragment` ]
    :param port_definitions: EquipmentPortDefinitionFragments sequence
    :type port_definitions: Sequence[ :class:`~psym.graphql.fragment.equipment_port_definition.EquipmentPortDefinitionFragment` ]
    """

    name: str
    category: Optional[str]
    id: str
    property_types: Sequence[PropertyDefinition]
    position_definitions: Sequence[EquipmentPositionDefinitionFragment]
    port_definitions: Sequence[EquipmentPortDefinitionFragment]


class EquipmentPortType(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Name
    :type name: str
    :param property_types: Property types sequence
    :type property_types: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param link_property_types: Link property types sequence
    :type link_property_types: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]
    """

    id: str
    name: str
    property_types: Sequence[PropertyDefinition]
    link_property_types: Sequence[PropertyDefinition]


class Equipment(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param external_id: External ID
    :type external_id: str, optional
    :param name: Name
    :type name: str
    :param equipment_type_name: Equipment type name
    :type equipment_type_name: str
    :param properties: PropertyFragment sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property.PropertyFragment` ])
    """

    id: str
    external_id: Optional[str]
    name: str
    equipment_type_name: str
    properties: Sequence[PropertyFragment]


class Link(NamedTuple):
    """
    :param id: Link ID
    :type id: str
    :param properties: Properties sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property.PropertyFragment` ]
    :param service_ids: Service IDs list
    :type service_ids: List[str]
    """

    id: str
    properties: Sequence[PropertyFragment]
    service_ids: List[str]


class EquipmentPortDefinitionAlias(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str, optional
    """

    name: str
    id: Optional[str] = None


class EquipmentPortDefinition(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str, optional
    :param visible_label: Visible label
    :type visible_label: str, optional
    :param port_definition_index: Index
    :type port_definition_index: int, optional
    :param port_type_name: Port type name
    :type port_type_name: str, optional
    :param connected_ports: ConnectedPorts list
    :type connected_ports: List [ :class:`~psym.common.data_class.EquipmentPortDefinitionAlias] , optional
    """

    name: str
    connected_ports: List[EquipmentPortDefinitionAlias]
    id: Optional[str] = None
    visible_label: Optional[str] = None
    port_definition_index: Optional[int] = None
    port_type_name: Optional[str] = None


class EquipmentPort(NamedTuple):
    """
    :param id: Equipment port ID
    :type id: str
    :param properties: Properties sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property.PropertyFragment` ]
    :param definition: EquipmentPortDefinition object
    :type definition: :class:`~psym.common.data_class.EquipmentPortDefinition`
    :param link: Link object
    :type link: :class:`~psym.common.data_class.Link`
    """

    id: str
    properties: Sequence[PropertyFragment]
    definition: EquipmentPortDefinition
    link: Optional[Link]


class Customer(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Name
    :type name: str
    :param external_id: External ID
    :type external_id: str, optional
    """

    id: str
    name: str
    external_id: Optional[str]

class KqiCategory(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class KqiPerspective(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class KqiTemporalFrecuency(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str
    
class KqiSource(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class Kqi (NamedTuple):
    """This function cannot be used because the mandatory parameters StarDatetime and EndDatetime are not allowed in the format required in the Api.
    
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param description: description
    :type description: str
    :param formula: formula
    :type formula: str
    :param startDatetime: startDatetime
    :type startDatetime: datetime
    :param EndDateTime: EndDateTime
    :type EndDateTime: datetime
    :param kqiSource: kqiSource Object
    :type kqiSource: `~psym.common.data_class.kqi_cource`
    :param kqiCategory: kqiCategory
    :type kqiCategory: `~psym.common.data_class.kqi_category`
    :param KqiPerspective: KqiPerspective
    :type KqiPerspective: `~psym.common.data_class.kqi_perspective`
    :param KqiTemporalFrecuency: KqiTemporalFrecuency
    :type KqiTemporalFrecuency: `~psym.common.data_class.kqi_temporal_frecuency`
    """

    id: str
    name: str
    description: str
    formula: str
    startDatetime: datetime
    EndDateTime: datetime
    kqiSource: str
    kqiCategory: str
    KqiPerspective: str
    KqiTemporalFrecuency: str

class KqiTarget (NamedTuple):
    """This function cannot be used because the mandatory parameters Starttime and Endtime are not allowed in the format required in the Apiand need the kqi entity.
    
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param frame: Frame
    :type frame: int
    :param allowed variation: Allowed variation
    :type allowed variation: int
    :param starttime: starttime
    :type starttime: time
    :param EndTime: EndTime
    :type EndTime: time
    :param impact: impact
    :type impact: str
    :param status: status
    :type status: str
    :param period: period
    :type period: str
    :param kqi: kqi Object
    :type kqi: `~psym.common.data_class.kqi`

    """

    id: str
    name: str
    frame: int
    allowedVariation: float
    starttime: time
    EndTime: time
    impact: str
    status: bool
    period: int
    kqi: str

class KqiComparator (NamedTuple):
    """This function cannot be used because it requires the kqiTarget entity, and this entity cannot be used because of a bug in the time parameters.he
    
    :param number: Number
    :type number: int
    :param id: ID
    :type id: str
    :param type comparator: type comparator
    :type type comparator: str
    :param kqiTraget: kqiTraget Object
    :type kqiTraget: `~psym.common.data_class.kqi_traget`
    :param comparator: comparator Object
    :type comparator: `~psym.common.data_class.comparator`
    """
    id: str
    number: int
    typeComparator:str
    kqiTraget: str
    comparator: str


class AlarmFiltering (NamedTuple):
    """This function cannot be used because it requires the kqiTarget entity, and this entity cannot be used because of a bug in the time parameters.he
    
    :param number: Number
    :type number: int
    :param id: ID
    :type id: str
    :param beginTime: beginTime
    :type beginTime: time
    :param endTime: endTime
    :type endTime: time
    :param reason: reason
    :type reason: str
    :param user: user
    :type user: str
    :param creationTime: creationTime
    :type creationTime: datetime
    :param alarmStatus: alarm status
    :type alarmStatus: `~psym.common.data_class.alarm_status`
    """
    id: str
    name: int
    beginTime: time
    endTime: time
    reason: str
    user: str
    creationTime: datetime
    alarmStatus: int



class domain (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class tech (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param domainFk: domain Object
    :type domainFk: psym.common.data_class.domain
    """

    id: str
    name: str
    domainFK: str

class KpiCategory(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class NetworkType (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class Kpi (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param description: description
    :type description: str
    :param status: status
    :type status: bool
    :param domainFk: domain Object
    :type domainFk: psym.common.data_class.domain
    :param kpiCategoryFK: str
    :type kpiCategoryFK: psym.common.data_class.kpi_category
    """

    id: str
    name: str
    description: str
    status: bool
    domain: str
    kpiCategory: str

class CounterFamily(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class Vendor (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class formula (NamedTuple):
    """
    :param textFormula: textFormula
    :type textFormula: str
    :param id: ID
    :type id: str
    :param description: description
    :type description: str
    :param status: status
    :type status: bool
    :param techFk: domain Object
    :type techFk: psym.common.data_class.tech
    :param networkTypeFk: str
    :type networkTypeFk: psym.common.data_class.network_type
    :param kpiFk: str
    :type kpiFk: psym.common.data_class.kpi
    """

    id: str
    textFormula: str
    status: bool
    techFk: str
    networkTypeFk: str
    kpiFk: str

class counter (NamedTuple):
    """
    :param name: name
    :type name: str
    :param id: ID
    :type id: str
    :param externalID: externalID
    :type externalID: str
    :param networkManagerSystem: networkManagerSystem
    :type networkManagerSystem: str
    :param counterFamilyFk: domain Object
    :type counterFamilyFk: psym.common.data_class.counter_family
    :param vendorFk: str
    :type vendorFk: psym.common.data_class.vendor
    """

    id: str
    name: str
    externalID: str
    networkManagerSystem: str
    counterFamilyFk: str
    vendorFk: str
    
class counterFormula (NamedTuple):
    """
    :param mandatory: mandatory
    :type mandatory: str
    :param id: ID
    :type id: str
    ;param counterFk: str
    :type counterFk: psym.common.data_class.counter
    :param formulaFk: str
    :type formulaFk: psym.common.data_class.formula
    """

    id: str
    mandatory: bool
    counterFk: str
    formulaFk: str

class eventSeverity (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class ruleType (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class comparator (NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class threshold (NamedTuple):
    """
    :param name: name
    :type name: str
    :param id: ID
    :type id: str
    :param description: description
    :type description: str
    :param status: status
    :type status: bool
    :param kpi: str
    :type kpi: `~psym.common.data_class.kpi`
    """

    id: str
    name: str
    description: str
    status: bool
    kpi: str

class rule (NamedTuple):
    """
    :param name: name
    :type name: str
    :param id: ID
    :type id: str
    :param gracePeriod: gracePeriod
    :type gracePeriod: str
    :param eventTypeName: eventTypeName
    :type eventTypeName: str
    :param specificProblem: specificProblem
    :type specificProblem: str
    :param additionalInfo: additionalInfo
    :type additionalInfo: str
    :param status: status
    :type status: bool
    :param ruleType: str
    :type ruleType: `~psym.common.data_class.rule_type`
    :param eventSeverity: str
    :type eventSeverity: `~psym.common.data_class.event_severity`
    :param threshold: str
    :type threshold: `~psym.common.data_class.threshold`
    """

    id: str
    name: str
    gracePeriod: int
    ruleType: str
    eventTypeName: str
    specificProblem: str
    additionalInfo: str
    status: bool
    eventSeverity: str
    threshold: str

class ruleLimit (NamedTuple):
    """
    :param number: number
    :type number: str
    :param id: ID
    :type id: str
    :param limitType: limitType
    :type limitType: str
    :param rule: rule
    :type rule: `~psym.common.data_class.rule`
    :param comparator: comparator
    :type comparator: `~psym.common.data_class.comparator`
    """

    id: int
    number: str
    limitType: str
    rule: str
    comparator: str
    

class alarmStatus(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str


class organization(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param description: Description
    :type description: str
    """

    id: str
    name: str
    description: str 

class Service(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param external_id: External ID
    :type external_id: str, optional
    :param service_type_name: Existing service type name
    :type service_type_name: str
    :param customer: Customer object
    :type customer: :class:`~psym.common.data_class.Customer`, optional
    :param properties: Properties sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property..PropertyFragment` ]
    """

    id: str
    name: str
    external_id: Optional[str]
    service_type_name: str
    customer: Optional[Customer]
    properties: Sequence[PropertyFragment]


class ServiceEndpointDefinition(NamedTuple):
    """
    :param id: ID
    :type id: str, optional
    :param name: Name
    :type name: str
    :param endpoint_definition_index: Index
    :type endpoint_definition_index: int
    :param role: Role
    :type role: str, optional
    :param equipment_type_id: Equipment type ID
    :type equipment_type_id: str
    """

    id: Optional[str]
    name: str
    endpoint_definition_index: int
    role: Optional[str]
    equipment_type_id: str


class ServiceEndpoint(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param equipment_id: Existing equipment ID
    :type equipment_id: str
    :param service_id: Existing service ID
    :type service_id: str
    :param definition_id: Existing service endpoint definition ID
    :type definition_id: str
    """

    id: str
    equipment_id: str
    service_id: str
    definition_id: str


class ServiceType(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    :param has_customer: Customer existence flag
    :type has_customer: bool
    :param property_types: PropertyDefinitions sequence
    :type property_types: Sequence[ :c;ass:`~psym.common.data_class.PropertyDefinition` ]
    :param endpoint_definitions: ServiceEndpointDefinitions list
    :type endpoint_definitions: List[ :class:`~psym.common.data_class.ServiceEndpointDefinition` ]
    """

    name: str
    id: str
    has_customer: bool
    property_types: Sequence[PropertyDefinition]
    endpoint_definitions: List[ServiceEndpointDefinition]


class Document(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Name
    :type name: str
    :param parent_id: Parent ID
    :type parent_id: str
    :param parent_entity: Parent entity
    :type parent_entity: :class:`~psym.graphql.enum.image_entity.ImageEntity`
    :param category: Category
    :type category: str, optional
    """

    id: str
    name: str
    parent_id: str
    parent_entity: ImageEntity
    category: Optional[str]
    document_category: Optional[DocumentCategory]


class User(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param auth_id: auth ID
    :type auth_id: str
    :param email: email
    :type email: str
    :param status: status
    :type status: :class:`~psym.graphql.enum.user_role.UserStatus`
    :param role: role
    :type role: :class:`~psym.graphql.enum.user_status.UserRole`
    :param organization: organization
    :type organization: :class:`~psym.graphql.data_class.organization.organization`
    """

    id: str
    auth_id: str
    email: str
    firstName: str
    lastName: str
    status: UserStatus
    role: UserRole
    organization: Optional[organization]

class RecommendationSources(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class RecommendationsCategory(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param id: ID
    :type id: str
    """

    id: str
    name: str

class recommendations (NamedTuple):
    """
    :param externalID: externalID
    :type externalID: str
    :param id: ID
    :type id: str
    :param resource: resource
    :type resource: str
    :param shortDescription: shortDescription
    :type shortDescription: str
    :param LongDescription: LongDescription
    :type LongDescription: str
    :param command: command
    :type command: str
    :param runbook: runbook
    :type runbook: str
    :param priority: priority
    :type priority: str
    :param status: status
    :type status: bool
    :param used: used
    :type used: int
    :param vendor: vendor
    :type vendor: str
    :param RecomendationSources: RecomendationSources
    :type RecomendationSources::`~psym.graphql.enum.RecomendationSources
    :param RecomendationsCategory: RecomendationsCategory
    :type RecomendationsCategory: `~psym.graphql.enum.RecomendationsCategory
    :param userCreated: userCreated
    :type userCreated:`~psym.graphql.enum.user`
    :param userApproved: userApproved
    :type userApproved:`~psym.graphql.enum.user`

    """

    id: int
    externalID: str
    resource: str
    alarmType: str
    shortDescription: str
    LongDescription: str
    command: str
    runbook: str
    priority: str
    status: bool
    used: int
    vendor: str
    RecomendationSources: str
    RecomendationsCategory: str
    userCreated: int
    userApproved: int


class SiteSurvey(NamedTuple):
    """
    :param name: Name
    :type name: str
    :param survey_id: ID
    :type survey_id: str
    :param completion_time: Complition time
    :type completion_time: datetime
    :param source_file_id: Source file ID
    :type source_file_id: str, optional
    :param source_file_name: Source file name
    :type source_file_name: str, optional
    :param source_file_key: Source file key
    :type source_file_key: str, optional
    :param forms: Forms
    :type forms: Dict[str, Dict[str, Any]]
    """

    name: str
    survey_id: str
    completion_time: datetime
    source_file_id: Optional[str]
    source_file_name: Optional[str]
    source_file_key: Optional[str]
    forms: Dict[str, Dict[str, Any]]


class WorkOrderType(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Work order type name
    :type name: str
    :param description: Work order type description
    :type description: str, optional
    :param property_types: PropertyTypes list
    :type property_types: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]
    """

    id: str
    name: str
    description: Optional[str]
    property_types: Sequence[PropertyDefinition]


class WorkOrderDefinition(NamedTuple):
    """
    :param definition_index: Work order definition index
    :type definition_index: int, optional
    :param work_order_type_id: Work order type ID
    :type work_order_type_id: str
    :param id: ID
    :type id: str, optional
    """

    definition_index: Optional[int]
    work_order_type_id: str
    id: Optional[str] = None


class WorkOrder(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Name
    :type name: str
    :param description: Description
    :type description: str, optional
    :param work_order_type_name: Existing work order type name
    :type work_order_type_name: str
    :param location_id: Existing location ID
    :type location_id: str, optional
    :param project_id: Existing project ID
    :type project_id: str, optional
    :param properties: PropertyFragment sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property.PropertyFragment` ])
    :param owner_id: Existing user ID, work order owner
    :type owner_id: str, optional
    :param assignee_id: Existing user ID, assigned to work order
    :type assignee_id: str, optional
    :param status: Work order status
    :type status: :class:`~psym.graphql.enum.work_order_status.WorkOrderStatus`, optional
    :param priority: Work order priority
    :type priority: :class:`~psym.graphql.enum.work_order_priority.WorkOrderPriority`, optional
    :param organization: organization
    :type organization: :class:`~psym.graphql.data_class.organization`
    """

    id: str
    name: str
    description: Optional[str]
    work_order_type_name: str
    location_id: Optional[str]
    project_id: Optional[str]
    properties: Sequence[PropertyFragment]
    owner_id: Optional[str]
    assignee_id: Optional[str]
    status: Optional[WorkOrderStatus]
    priority: Optional[WorkOrderPriority]
    organization:Optional[str]


class ProjectType(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Project type name
    :type name: str
    :param description: Project type description
    :type description: str, optional
    :param property_types: PropertyTypes sequence
    :type property_types: Sequence[ :class:`~psym.common.data_class.PropertyDefinition` ]
    :param work_order_definitions: WorkOrderDefinitions list
    :type work_order_definitions: List[ :class:`~psym.common.data_class.WorkOrderDefinition` ]
    """

    id: str
    name: str
    description: Optional[str]
    property_types: Sequence[PropertyDefinition]
    work_order_definitions: List[WorkOrderDefinition]


class Project(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Project name
    :type name: str
    :param description: Project description
    :type description: str, optional
    :param priority: Project priority
    :type priority: :class:`~psym.graphql.enum.project_priority.ProjectPriority`
    :param created_by: Existing user ID, project creator
    :type created_by: str, optional
    :param project_type_name: Existing project type name
    :type project_type_name: str
    :param project_type_id: Existing project type ID
    :type project_type_id: str
    :param location_id: Existing location ID
    :type location_id: str, optional
    :param work_orders: WorkOrders list
    :type work_orders: List[ :class:`~psym.common.data_class.WorkOrder` ]
    :param properties: PropertyFragment sequence
    :type properties: Sequence[ :class:`~psym.graphql.fragment.property.PropertyFragment` ]
    """

    id: str
    name: str
    description: Optional[str]
    priority: ProjectPriority
    created_by: Optional[str]
    project_type_name: str
    project_type_id: str
    location_id: Optional[str]
    work_orders: List[WorkOrder]
    properties: Sequence[PropertyFragment]


class FlowDraft(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Flow draft name
    :type name: str
    """

    id: str
    name: str


class Flow(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param name: Flow name
    :type name: str
    """

    id: str
    name: str


class FlowInstance(NamedTuple):
    """
    :param id: ID
    :type id: str
    :param status: status
    :type status: class:`~psym.graphql.enum.flow_instance_status.FlowInstanceStatus`
    """

    id: str
    status: FlowInstanceStatus



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

from typing import Dict, Iterator, MutableMapping, TypeVar

from psym.exceptions import EntityNotFoundError

from .data_class import (
    EquipmentPortType,
    EquipmentType,
    LocationType,
    ProjectType,
    ServiceType,
    WorkOrderType,
)
from .data_enum import Entity

T = TypeVar("T")


class Cache(MutableMapping[str, T]):
    def __init__(self, entity: Entity) -> None:
        self.store: Dict[str, T] = {}
        self.entity = entity

    def __getitem__(self, key: str) -> T:
        if key not in self.store:
            raise EntityNotFoundError(entity=self.entity, entity_name=key)
        return self.store[self.__keytransform__(key)]

    def __setitem__(self, key: str, value: T) -> None:
        self.store[self.__keytransform__(key)] = value

    # pyre-fixme[14]: `__delitem__` overrides method defined in `MutableMapping`
    #  inconsistently.
    def __delitem__(self, key: str) -> None:
        del self.store[self.__keytransform__(key)]

    def __iter__(self) -> Iterator[str]:
        return iter(self.store)

    def __len__(self) -> int:
        return len(self.store)

    def __keytransform__(self, key: str) -> str:
        return key


# pyre-fixme[16]: `Cache` has no attribute `__getitem__`.
LOCATION_TYPES = Cache[LocationType](Entity.LocationType)
EQUIPMENT_TYPES = Cache[EquipmentType](Entity.EquipmentType)
SERVICE_TYPES = Cache[ServiceType](Entity.ServiceType)
PORT_TYPES = Cache[EquipmentPortType](Entity.EquipmentPortType)
WORK_ORDER_TYPES = Cache[WorkOrderType](Entity.WorkOrderType)
PROJECT_TYPES = Cache[ProjectType](Entity.ProjectType)


def clear_types() -> None:
    LOCATION_TYPES.clear()
    EQUIPMENT_TYPES.clear()
    SERVICE_TYPES.clear()
    PORT_TYPES.clear()
    WORK_ORDER_TYPES.clear()
    PROJECT_TYPES.clear()



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.edit_rule_limit_input import EditRuleLimitInput


# fmt: off
QUERY: List[str] = ["""
mutation editRuleLimit($input: EditRuleLimitInput!) {
  editRuleLimit(input: $input) {
    id
    number
    limitType
    comparator{id}
    rule{id}
  }
}


"""
]


class editRuleLimit:
    @dataclass(frozen=True)
    class editRuleLimitData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class RuleLimit(DataClassJsonMixin):
            @dataclass(frozen=True)
            class Comparator(DataClassJsonMixin):
                id: str

            @dataclass(frozen=True)
            class Rule(DataClassJsonMixin):
                id: str

            id: str
            number: int
            limitType: str
            comparator: Comparator
            rule: Optional[Rule]

        editRuleLimit: RuleLimit

    # fmt: off
    @classmethod
    def execute(cls, client: Client, input: EditRuleLimitInput) -> editRuleLimitData.RuleLimit:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.editRuleLimitData.from_dict(response_text)
        return res.editRuleLimit

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, input: EditRuleLimitInput) -> editRuleLimitData.RuleLimit:
        variables: Dict[str, Any] = {"input": input}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.editRuleLimitData.from_dict(response_text)
        return res.editRuleLimit



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..input.counter_filter_input import CounterFilterInput
from ..input.counter_order import CounterOrder


# fmt: off
QUERY: List[str] = ["""
query counters(
$after: Cursor
$first: Int
$before: Cursor
$last: Int
$orderBy: CounterOrder
$filterBy: [CounterFilterInput!]
){
counters(after: $after, first: $first, before: $before, last: $last, orderBy: $orderBy, filterBy: $filterBy){
totalCount
edges{
node{
id
name
externalID
networkManagerSystem
vendorFk{id}
counterFamily{id}

}
      cursor
    }
    pageInfo{
      hasNextPage
      hasPreviousPage
      startCursor
      endCursor
    }
  }
}
"""
]


class counters:
    @dataclass(frozen=True)
    class countersData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class CounterConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class CounterEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Counter(DataClassJsonMixin):
                    @dataclass(frozen=True)
                    class Vendor(DataClassJsonMixin):
                        id: str

                    @dataclass(frozen=True)
                    class CounterFamily(DataClassJsonMixin):
                        id: str

                    id: str
                    name: str
                    externalID: str
                    networkManagerSystem: str
                    vendorFk: Vendor
                    counterFamily: Optional[CounterFamily]

                node: Optional[Counter]
                cursor: str

            @dataclass(frozen=True)
            class PageInfo(DataClassJsonMixin):
                hasNextPage: bool
                hasPreviousPage: bool
                startCursor: Optional[str]
                endCursor: Optional[str]

            totalCount: int
            edges: List[CounterEdge]
            pageInfo: PageInfo

        counters: CounterConnection

    # fmt: off
    @classmethod
    def execute(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[CounterOrder] = None, filterBy: List[CounterFilterInput] = []) -> countersData.CounterConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.countersData.from_dict(response_text)
        return res.counters

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, after: Optional[str] = None, first: Optional[int] = None, before: Optional[str] = None, last: Optional[int] = None, orderBy: Optional[CounterOrder] = None, filterBy: List[CounterFilterInput] = []) -> countersData.CounterConnection:
        variables: Dict[str, Any] = {"after": after, "first": first, "before": before, "last": last, "orderBy": orderBy, "filterBy": filterBy}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.countersData.from_dict(response_text)
        return res.counters



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.

import datetime
import json
import warnings
from typing import Callable, Dict, List, Mapping, Optional, Sequence, Tuple, Union, cast

from dacite import Config, from_dict

from psym.common.data_class import (
    TYPE_AND_FIELD_NAME,
    DataTypeName,
    DocumentCategory,
    PropertyDefinition,
    PropertyValue,
    ReturnType,
)
from psym.common.data_enum import Entity
from psym.graphql.input.document_category_input import DocumentCategoryInput

from .exceptions import EntityNotFoundError
from .graphql.enum.property_kind import PropertyKind
from .graphql.fragment.equipment_port_definition import EquipmentPortDefinitionFragment
from .graphql.fragment.equipment_position_definition import (
    EquipmentPositionDefinitionFragment,
)
from .graphql.fragment.property import PropertyFragment
from .graphql.input.equipment_port_input import EquipmentPortInput
from .graphql.input.equipment_position_input import EquipmentPositionInput
from .graphql.input.property_input import PropertyInput
from .graphql.input.property_type_input import PropertyTypeInput
from .graphql.input.equipment_port_connection_input import EquipmentPortConnectionInput


def format_to_type_and_field_name(type_key: str) -> Optional[DataTypeName]:
    formated = TYPE_AND_FIELD_NAME.get(type_key, None)
    return formated


def get_graphql_input_field(
    property_type_name: str, type_key: str, value: PropertyValue
) -> Dict[str, PropertyValue]:
    formated_type = format_to_type_and_field_name(type_key)
    if formated_type is None:
        raise Exception(
            f"property type {property_type_name} has not supported type {type_key}"
        )
    if type_key in ["string", "enum"]:
        assert isinstance(value, str) or isinstance(
            value, bytes
        ), f"property {property_type_name} is not of type {type_key}"
    elif type_key == "date":
        assert isinstance(
            value, datetime.date
        ), f"property {property_type_name} is not of type {type_key}"
        value = datetime.datetime.strftime(
            datetime.datetime.combine(value, datetime.datetime.min.time()), "%Y-%m-%d"
        )
    elif type_key == "datetime_local":
        assert isinstance(
            value, datetime.datetime
        ), f"property {property_type_name} is not of type {type_key}"
        value = datetime.datetime.strftime(value, "%Y-%m-%d %H:%M:%S")
    elif type_key == "gps_location":
        assert isinstance(
            value, tuple
        ), f"property {property_type_name} is not of type {type_key}"
        gps_value = value
        assert (
            len(gps_value) == 2
            and isinstance(gps_value[0], float)
            and isinstance(gps_value[1], float)
        ), f"property {property_type_name} is not of type tuple(float, float)"
        return {
            formated_type.graphql_field_name[0]: gps_value[0],
            formated_type.graphql_field_name[1]: gps_value[1],
        }
    else:
        assert isinstance(
            value, formated_type.data_type
        ), f"property {property_type_name} is not of type {type_key}"

    return {formated_type.graphql_field_name[0]: cast(PropertyValue, value)}


def update_property_input_ids(
    existing_properties: Dict[str, PropertyFragment],
    property_inputs: List[PropertyInput],
) -> List[PropertyInput]:
    new_property_inputs = []
    for p in property_inputs:
        input_id = p.id
        if p.propertyTypeID in existing_properties:
            input_id = existing_properties[p.propertyTypeID].id
        new_property_inputs.append(
            PropertyInput(
                propertyTypeID=p.propertyTypeID,
                id=input_id,
                stringValue=p.stringValue,
                intValue=p.intValue,
                booleanValue=p.booleanValue,
                floatValue=p.floatValue,
                latitudeValue=p.latitudeValue,
                longitudeValue=p.longitudeValue,
                rangeFromValue=p.rangeFromValue,
                rangeToValue=p.rangeToValue,
                nodeIDValue=p.nodeIDValue,
                isEditable=p.isEditable,
                isInstanceProperty=p.isInstanceProperty,
            )
        )
    return new_property_inputs


def get_graphql_property_type_inputs(
    property_types: Sequence[PropertyDefinition],
    properties_dict: Mapping[str, PropertyValue],
) -> List[PropertyTypeInput]:
    """This function gets existing property types and dictionary, where key - are type names, and keys - new values
    formats data, validates existence of keys from `properties_dict` in `property_types` and returns list of PropertyTypeInput

        Args:
            property_types (List[ `psym.common.data_class.PropertyDefinition` ]): list of existing property types
            properties_dict (Dict[str, PropertyValue]): dictionary of properties, where
            - str - name of existing property
            - PropertyValue - new value of existing type for this property

        Returns:
            List[ `psym.graphql.input.property_type.PropertyTypeInput` ]

        Raises:
            `psym.exceptions.EntityNotFoundError`: if there any unknown property name in `properties_dict` keys
    """
    properties = []
    property_type_names = {}
    for property_type in property_types:
        property_type_names[property_type.property_name] = property_type
    for name, value in properties_dict.items():
        if name not in property_type_names:
            raise EntityNotFoundError(entity=Entity.PropertyType, entity_name=name)
        property_type_id = property_type_names[name].id
        assert property_type_id is not None, f"property {name} has no id"
        assert not property_type_names[name].is_fixed, f"property {name} is fixed"
        result: Dict[str, Union[PropertyValue, PropertyKind]] = {
            "id": property_type_id,
            "name": name,
            "type": PropertyKind(property_type_names[name].property_kind),
        }
        result.update(
            get_graphql_input_field(
                property_type_name=name,
                type_key=property_type_names[name].property_kind.value,
                value=value,
            )
        )
        properties.append(
            from_dict(
                data_class=PropertyTypeInput, data=result, config=Config(strict=True)
            )
        )

    return properties


def get_graphql_document_category_inputs(
    document_categories: Sequence[DocumentCategory],
    categories_name_dict: Mapping[str, str],
) -> List[DocumentCategoryInput]:
    """This function gets existing document categories and dictionary, where key - are current names, and keys - new name,
    validates existence of keys from `categories_name_dict` in `document_categories` and returns list of DocumentCategoryInput

        Args:
            document_categories (List[ `psym.common.data_class.DocumentCategory` ]): list of existing document categories
            categories_name_dict (Dict[str, str]): dictionary of categories name, where
            - str - name of existing category
            - str - new document category name

        Returns:
            List[ `psym.graphql.input.document_category.DocumentCategoryInput` ]

        Raises:
            `psym.exceptions.EntityNotFoundError`: if there any unknown category name in `categories_name_dict` keys
    """
    document_categories_list = []
    document_category_names = {}
    for doc_category in document_categories:
        document_category_names[doc_category.name] = doc_category
    for name, value in categories_name_dict.items():
        if name not in document_category_names:
            raise EntityNotFoundError(entity=Entity.DocumentCategory, entity_name=name)
        document_category_id = document_category_names[name].id
        document_category_index = document_category_names[name].dc_index
        assert document_category_id is not None, f"document category {name} has no id"
        result: Dict[str, str] = {
            "id": document_category_id,
            "index": document_category_index,
            "name": value,
        }

        document_categories_list.append(
            from_dict(
                data_class=DocumentCategoryInput,
                data=result,
                config=Config(strict=True),
            )
        )

    return document_categories_list


def get_graphql_property_inputs(
    property_types: Sequence[PropertyDefinition],
    properties_dict: Mapping[str, PropertyValue],
) -> List[PropertyInput]:
    """This function gets existing property types and dictionary, where key - are type names, and keys - new values
    formats data, validates existence of keys from `properties_dict` in `property_types` and returns list of PropertyInput

        Args:
            property_types (Sequence[ `psym.common.data_class.PropertyDefinition` ]): list of existing property types
            properties_dict (Mapping[str, PropertyValue]): dictionary of properties, where
                str: name of existing property
                PropertyValue: new value of existing type for this property

        Returns:
            List[ `psym.graphql.input.property.PropertyInput` ]

        Raises:
            `psym.exceptions.EntityNotFoundError`: if there any unknown property name in properties_dict keys

        Example:
            ```
            property_types = LOCATION_TYPES[location_type].property_types
            properties = get_graphql_property_inputs(property_types, properties_dict)
            ```
    """
    properties = []
    property_type_names = {}

    for property_type in property_types:
        property_type_names[property_type.property_name] = property_type

    for name, value in properties_dict.items():
        if name not in property_type_names:
            raise EntityNotFoundError(entity=Entity.PropertyType, entity_name=name)
        property_type_id = property_type_names[name].id
        assert property_type_id is not None, f"property {name} has no id"
        assert not property_type_names[name].is_fixed, f"property {name} is fixed"
        pk = property_type_names[name].property_kind
        if pk == PropertyKind.enum:
            def_raw_val = property_type_names[name].default_raw_value
            if def_raw_val is None:
                def_raw_val = ""
            values = json.loads(def_raw_val)
            assert value in values, f"{value} is not in {values}"
        result: Dict[str, PropertyValue] = {"propertyTypeID": property_type_id}
        result.update(
            get_graphql_input_field(
                property_type_name=name,
                type_key=property_type_names[name].property_kind.value,
                value=value,
            )
        )
        properties.append(
            from_dict(data_class=PropertyInput, data=result, config=Config(strict=True))
        )

    return properties


def _get_property_value(
    property_type: str, property: PropertyFragment
) -> Tuple[PropertyValue, ...]:
    formated_name = format_to_type_and_field_name(property_type)
    if formated_name is None:
        raise AssertionError(f"Unknown property type - {property_type}")

    str_fields = formated_name.graphql_field_name
    values = []
    for str_field in str_fields:
        field_data = property.__dict__[str_field]
        if property_type == "date":
            values.append(
                datetime.datetime.strptime(cast(str, field_data), "%Y-%m-%d").date()
            )
        elif property_type == "datetime_local":
            values.append(
                datetime.datetime.strptime(cast(str, field_data), "%Y-%m-%d %H:%M:%S")
            )
        else:
            values.append(field_data)
    return tuple(value for value in values)


def get_position_definition_input(
    position_definition: EquipmentPositionDefinitionFragment, is_new: bool = True
) -> EquipmentPositionInput:
    return EquipmentPositionInput(
        name=position_definition.name,
        id=position_definition.id if not is_new else None,
        index=position_definition.index,
        visibleLabel=position_definition.visibleLabel,
    )


def get_port_definition_input(
    port_definition: EquipmentPortDefinitionFragment, is_new: bool = True
) -> EquipmentPortInput:
    return EquipmentPortInput(
        name=port_definition.name,
        id=port_definition.id if not is_new else None,
        index=port_definition.index,
        visibleLabel=port_definition.visibleLabel,
        portTypeID=port_definition.portType.id if port_definition.portType else None,
        connectedPorts=[
            EquipmentPortConnectionInput(id=port.id, name=port.name)
            for port in port_definition.connectedPorts
        ],
    )


def deprecated(
    deprecated_in: str,
    deprecated_by: str
    # pyre-fixme[34]: `Variable[ReturnType]` isn't present in the function's parameters.
) -> Callable[[Callable[..., ReturnType]], Callable[..., ReturnType]]:
    def wrapped(func: Callable[..., ReturnType]) -> Callable[..., ReturnType]:
        def wrapper(*args: str, **kwargs: int) -> Callable[..., ReturnType]:
            func_name = func.__name__
            message = f"{func_name} is deprecated in {deprecated_in}. Use the {deprecated_by} function instead."
            warnings.warn(message, DeprecationWarning, stacklevel=2)
            return cast(Callable[..., ReturnType], func(*args, **kwargs))

        return cast(Callable[..., ReturnType], wrapper)

    return wrapped



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class LocationFilterType(Enum):
    LOCATION_INST = "LOCATION_INST"
    LOCATION_INST_NAME = "LOCATION_INST_NAME"
    LOCATION_INST_EXTERNAL_ID = "LOCATION_INST_EXTERNAL_ID"
    LOCATION_TYPE = "LOCATION_TYPE"
    LOCATION_INST_HAS_EQUIPMENT = "LOCATION_INST_HAS_EQUIPMENT"
    PROPERTY = "PROPERTY"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "LocationFilterType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from .property import PropertyFragment, QUERY as PropertyFragmentQuery

# fmt: off
QUERY: List[str] = PropertyFragmentQuery + ["""
fragment LocationFragment on Location {
  id
  name
  latitude
  longitude
  externalId
  locationType {
    name
  }
  properties {
    ...PropertyFragment
  }
}

"""]

@dataclass(frozen=True)
class LocationFragment(DataClassJsonMixin):
    @dataclass(frozen=True)
    class LocationType(DataClassJsonMixin):
        name: str

    @dataclass(frozen=True)
    class Property(PropertyFragment):
        pass

    id: str
    name: str
    latitude: Number
    longitude: Number
    externalId: Optional[str]
    locationType: LocationType
    properties: List[Property]



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from ...config import custom_scalars, datetime
from gql_client.runtime.variables import encode_variables
from gql import gql, Client
from gql.transport.exceptions import TransportQueryError
from functools import partial
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional
from time import perf_counter
from dataclasses_json import DataClassJsonMixin, config

from ..fragment.location import LocationFragment, QUERY as LocationFragmentQuery

from ..input.location_filter_input import LocationFilterInput


# fmt: off
QUERY: List[str] = LocationFragmentQuery + ["""
query LocationSearchQuery($filters: [LocationFilterInput!]!, $limit: Int) {
  locations(filterBy: $filters, first: $limit) {
    edges {
      node {
        ...LocationFragment
      }
    }
    totalCount
  }
}

"""
]


class LocationSearchQuery:
    @dataclass(frozen=True)
    class LocationSearchQueryData(DataClassJsonMixin):
        @dataclass(frozen=True)
        class LocationConnection(DataClassJsonMixin):
            @dataclass(frozen=True)
            class LocationEdge(DataClassJsonMixin):
                @dataclass(frozen=True)
                class Location(LocationFragment):
                    pass

                node: Optional[Location]

            edges: List[LocationEdge]
            totalCount: int

        locations: Optional[LocationConnection]

    # fmt: off
    @classmethod
    def execute(cls, client: Client, filters: List[LocationFilterInput] = [], limit: Optional[int] = None) -> Optional[LocationSearchQueryData.LocationConnection]:
        variables: Dict[str, Any] = {"filters": filters, "limit": limit}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = client.execute(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.LocationSearchQueryData.from_dict(response_text)
        return res.locations

    # fmt: off
    @classmethod
    async def execute_async(cls, client: Client, filters: List[LocationFilterInput] = [], limit: Optional[int] = None) -> Optional[LocationSearchQueryData.LocationConnection]:
        variables: Dict[str, Any] = {"filters": filters, "limit": limit}
        new_variables = encode_variables(variables, custom_scalars)
        response_text = await client.execute_async(
            gql("".join(set(QUERY))), variable_values=new_variables
        )
        res = cls.LocationSearchQueryData.from_dict(response_text)
        return res.locations



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class CheckListItemType(Enum):
    simple = "simple"
    string = "string"
    enum = "enum"
    files = "files"
    yes_no = "yes_no"
    cell_scan = "cell_scan"
    wifi_scan = "wifi_scan"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "CheckListItemType":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ServiceStatus(Enum):
    PENDING = "PENDING"
    IN_SERVICE = "IN_SERVICE"
    MAINTENANCE = "MAINTENANCE"
    DISCONNECTED = "DISCONNECTED"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ServiceStatus":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class KqiTemporalFrequencyOrderField(Enum):
    NAME = "NAME"
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "KqiTemporalFrequencyOrderField":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class FilterOperator(Enum):
    IS = "IS"
    IS_NIL = "IS_NIL"
    IS_NIL_OR_DATE_GREATER_OR_EQUAL_THAN = "IS_NIL_OR_DATE_GREATER_OR_EQUAL_THAN"
    CONTAINS = "CONTAINS"
    IS_ONE_OF = "IS_ONE_OF"
    IS_NOT_ONE_OF = "IS_NOT_ONE_OF"
    DATE_GREATER_THAN = "DATE_GREATER_THAN"
    DATE_LESS_THAN = "DATE_LESS_THAN"
    DATE_GREATER_OR_EQUAL_THAN = "DATE_GREATER_OR_EQUAL_THAN"
    DATE_LESS_OR_EQUAL_THAN = "DATE_LESS_OR_EQUAL_THAN"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "FilterOperator":
        return cls.MISSING_ENUM



#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import ruleType
from psym.exceptions import EntityNotFoundError

from ..graphql.input.edit_rule_type_input import EditRuleTypeInput
from ..graphql.input.add_rule_type_input import AddRuleTypeInput
from ..graphql.mutation.add_rule_type import addRuleType
from ..graphql.mutation.edit_rule_type import editRuleType
from ..graphql.mutation.remove_rule_type import removeRuleType
from ..graphql.query.rule_types import ruleTypes
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional
from psym.common.data_enum import Entity



def add_rule_type(
    client: SymphonyClient, name: str
) -> ruleType:
    """This function adds Rule Type.

    :param name: Rule Type name
    :type name: str

    :return: ruleType object
    :rtype: :class:`~psym.common.data_class.ruleType`)

    **Example 2**

    .. code-block:: python

        new_rule_type = client.add_rule_type(
            name="rule_type",

        )
    """
    rule_type_input = AddRuleTypeInput(name=name)
    if (len(name) != 0): 
        result = addRuleType.execute(client, input=rule_type_input)
        return ruleType(name=result.name, id=result.id)
    else:
        print ("you cant leave the name parameter empty")

def edit_rule_type(
    client: SymphonyClient,
    rule_type: ruleType,
    new_name: Optional[str] = None,
) -> None:
    """This function edits Rule Type.

    :param rule_type: Rule Type entity
    :type name: str
    :param new_name: Rule Type name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.ruleType`

    **Example 1**

    .. code-block:: python

        new_rule_type_Edited = client.edit_rule_type(
            rule_type=ruleType,
            new_name="rule_type_edited",

        )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editRuleType.execute(client, input=EditRuleTypeInput(id=rule_type.id, name=new_name))

def get_rule_types(client: SymphonyClient) -> Iterator[ruleType]:
    """ this funtion Get Rule Types


    :return: ruleType object
    :rtype: Iterator[ :class:`~psym.common.data_class.ruleType` ]

    **Example**

    .. code-block:: python

        rule_types = client.get_rule_types()
        for rule_type in rule_types:
            print(rule_type.name)
    """
    rule_types_ = ruleTypes.execute(client, first=PAGINATION_STEP)
    edges = rule_types_.edges if rule_types_ else []
    while rule_types_ is not None and rule_types_.pageInfo.hasNextPage:
        rule_types_ = ruleTypes.execute(
            client, after=rule_types_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if rule_types_ is not None:
            edges.extend(rule_types_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield ruleType(
                id=node.id,
                name=node.name,
            )

def remove_rule_type(client: SymphonyClient, id: str) -> None:
    """This function delete Rule Type.

    :param name: Rule Type name
    :type name: :class:`~psym.common.data_class.ruleType`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_rule_type(id=123456798)
    """
    removeRuleType.execute(client, id=id)





#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import  NetworkType, formula, Kpi, tech

from ..graphql.input.edit_formula_input import EditFormulaInput
from ..graphql.input.add_formula_input import AddFormulaInput
from ..graphql.mutation.add_formula import addFormula
from ..graphql.mutation.edit_formula import editFormula
from ..graphql.mutation.remove_formula import removeFormula
from ..graphql.query.formulas import formulas
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_formula(
    client: SymphonyClient, textFormula: str, status: bool,  tech: str, networkType: str,  kpi: str
) -> formula:
    """This function adds formula.

    :param textFormula: formula name
    :type textFormula: str
    :param description: description
    :type description: str
    :param status: status
    :type status: bool
    :param techFk: domain Object
    :type techFk: psym.common.data_class.tech
    :param networkTypeFk: str
    :type networkTypeFk: psym.common.data_class.network_type
    :param kpiFk: str
    :type kpiFk: psym.common.data_class.kpi

    :return: formula object
    :rtype: :class:`~psym.common.data_class.formula`

    **Example 1**

    .. code-block:: python

       domain = client.add_domain(
            name="domain",
        )

        kpi_category = client.add_kpi_category(
            name="kpi_category",
        )

        Kpi = client.add_Kpi(
            name="Kpi_2",
            description="new kpi",
            status=True,
            domain=domain.id,
            kpiCategory=kpi_category.id
        )

        domain = client.add_domain(
            name="domain",
        )

        tech = client.add_Tech(
            name="new_Tech", 
            domain=domain.id
            
        network_type = client.add_network_type(
            name="network_type",
        )

        new_formula = client.add_formula(
            textFormula="formula_1",
            status=True,
            kpi=Kpi.id,
            tech=tech.id,
            networkType=network_type.id
        )
    """
    formula_input = AddFormulaInput(
    textFormula=textFormula, 
    status=status, 
    techFk= tech,
    networkTypeFk= networkType,
    kpiFk= kpi)
    if (textFormula == "") or (textFormula == "") :
      print ("you cant leave the name parameter empty")
    elif textFormula!=str:
      print ("the input must be a text, example name=''new name''")
    else:
        result = addFormula.execute(client, input=formula_input)
        return formula(textFormula=result.textFormula, 
        id=result.id,  
        status=result.status, 
        techFk=result.techFk,
        networkTypeFk=result.networkTypeFk,
        kpiFk=result.kpiFk
)

def edit_formula(
    client: SymphonyClient,
    formula: formula,
    new_name: Optional[str] = None,
    status: bool = None,
    tech: tech = None,
    networkType: NetworkType = None,
    kpi: Kpi = None
) -> None:
    """This function edits formula.

    :param formula: formula entity
    :type name: str
    :param new_name: formula name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.formula`

    **Example 1**

    .. code-block:: python

        formula_edited = client.edit_formula(
            formula=formula,
            new_name="new_formula_edited",
            status=True,
            kpi=Kpi.id,
            tech=tech.id,
            networkType=network_type.id
            )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editFormula.execute(client, input=EditFormulaInput(
        id=formula.id, 
        textFormula=new_name,
        status= status,
        networkTypeFk=networkType,
        kpiFk=kpi,
        techFk=tech
        ))


def get_formulas(client: SymphonyClient) -> Iterator[Kpi]:
    """ this funtion Get formulas


    :return: formula object
    :rtype: Iterator[ :class:`~psym.common.data_class.formula` ]

    **Example**

    .. code-block:: python

        formulas = client.get_formulas()
        for formula in formulas:
            print(formula.name)
    """
    formulas_ = formulas.execute(client, first=PAGINATION_STEP)
    edges = formulas_.edges if formulas_ else []
    while formulas_ is not None and formulas_.pageInfo.hasNextPage:
        formulas_ = formulas.execute(
            client, after=formulas_.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if formulas_ is not None:
            edges.extend(formulas_.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield formula(
                id=node.id,
                textFormula=node.textFormula,
                status=node.status,
                networkTypeFk=node.networkTypeFk,
                kpiFk=node.kpiFk,
                techFk=node.techFk

            )


def remove_formula(client: SymphonyClient, id: str) -> None:
    """This function delete formula.

    :param name: formula name
    :type name: :class:`~psym.common.data_class.formula`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_formula(id=123456789)
    """
    removeFormula.execute(client, id=id)











#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.




from psym.client import SymphonyClient
from psym.common.data_class import KqiTemporalFrecuency

from ..graphql.input.edit_kqi_temporal_frequency_input import EditKqiTemporalFrequencyInput
from ..graphql.input.add_kqi_temporal_frequency_input import AddKqiTemporalFrequencyInput
from ..graphql.mutation.add_kqi_temporal_frecuency import AddKqiTemporalFrequencyInput, addKqiTemporalFrequency
from ..graphql.mutation.edit_kqi_temporal_frecuency import EditKqiTemporalFrequencyInput, editKqiTemporalFrequency
from ..graphql.query.kqi_temporal_frecuency import kqiTemporalFrequencies
from ..graphql.mutation.remove_kqi_temporal_frecuency import removeKqiTemporalFrequency
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_kqi_temporal_frecuency(
    client: SymphonyClient, name: str
) -> KqiTemporalFrecuency:
    """This function adds KqiTemporalFrecuency.

    :param name: KqiTemporalFrecuency name
    :type name: str

    :return: kqiTemporalFrecuency object
    :rtype: :class:`~psym.common.data_class.kqiTemporalFrecuency`

    **Example 2**

    .. code-block:: python

        new_kqi_temporal_frecuency = client.add_kqi_temporal_frecuency(
            name="kqi_temporal_frecuency",

        )
    """
    kpi_category_input = AddKqiTemporalFrequencyInput(name=name)
    if (len(name) != 0):
        result = addKqiTemporalFrequency.execute(client, input=kpi_category_input)
        return KqiTemporalFrecuency(name=result.name, id=result.id)
    else:
        print ("you cant leave the name parameter empty")

def edit_kqi_temporal_frecuency(
    client: SymphonyClient,
    kqitemporalfrecuency: KqiTemporalFrecuency,
    new_name: Optional[str] = None,
) -> None:
    """This function edits KqiTemporalFrecuency.

    :param kqi_temporal_frecuency: KqiTemporalFrecuency entity
    :type name: str
    :param new_name: KqiTemporalFrecuency name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.kqiTemporalFrecuency`

    **Example 1**

    .. code-block:: python

        new_kqi_temporal_frecuency_edited = client.edit_kqi_temporal_frecuency(
            kqi_temporal_frecuency=kqiTemporalFrecuency,
            new_name="kqi_temporal_frecuency_edited",

        )
    """
    params: Dict[str, Any] = {}
    if new_name is not None:
        params.update({"_name_": new_name})
    if new_name is not None:
        editKqiTemporalFrequency.execute(client, input=EditKqiTemporalFrequencyInput(id=kqitemporalfrecuency.id, name=new_name))


def get_kqi_temporal_frecuencies(client: SymphonyClient) -> Iterator[KqiTemporalFrecuency]:
    """ this funtion Get KqiTemporalFrecuencies


    :return: kqiTemporalFrecuency object
    :rtype: Iterator[ :class:`~psym.common.data_class.kqiTemporalFrecuency` ]

    **Example**

    .. code-block:: python

        kqi_temporal_frecuencyes = client.get_kqi_temporal_frecuencies()
        for kqi_temporal_frecuency in kqi_temporal_frecuencies:
            print(kqi_temporal_frecuency.name)
    """
    kqi_temporal_frecuencies = kqiTemporalFrequencies.execute(client, first=PAGINATION_STEP)
    edges = kqi_temporal_frecuencies.edges if kqi_temporal_frecuencies else []
    while kqi_temporal_frecuencies is not None and kqi_temporal_frecuencies.pageInfo.hasNextPage:
        kqi_temporal_frecuencies = kqiTemporalFrequencies.execute(
            client, after=kqi_temporal_frecuencies.pageInfo.endCursor, first=PAGINATION_STEP
        )
        if kqi_temporal_frecuencies is not None:
            edges.extend(kqi_temporal_frecuencies.edges)

    for edge in edges:
        node = edge.node
        if node is not None:
            yield KqiTemporalFrecuency(
                id=node.id,
                name=node.name,
            )


def delete_kqi_temporal_frecuency(client: SymphonyClient, id: str) -> None:
    """This function delete KqiTemporalFrecuency.

    :param name: KqiTemporalFrecuency name
    :type name: :class:`~psym.common.data_class.kqiTemporalFrecuency`
    :rtype: None

    **Example**

    .. code-block:: python

        client.delete_kqi_temporal_frecuency(id=123456789)
    """
    removeKqiTemporalFrequency.execute(client, id=id)






#!/usr/bin/env python3
# Copyright (c) 2004-present Facebook All rights reserved.
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file.


from psym.client import SymphonyClient
from psym.common.data_class import recommendations

from ..graphql.input.edit_recommendations_input import EditRecommendationsInput
from ..graphql.input.add_recommendations_input import AddRecommendationsInput
from ..graphql.mutation.add_recommendations import addRecommendations
from ..graphql.mutation.edit_recommendations import editRecommendations
from ..graphql.mutation.remove_recommendations import removeRecommendations
from ..graphql.query.recommendations import Recommendations
from psym.common.constant import PAGINATION_STEP
from typing import Any, Dict, Iterator, List, Optional




def add_recommendations(
    client: SymphonyClient, 
    externalID: str,
    resource: str,
    alarmType: str,
    shortDescription: str,
    LongDescription: str,
    command: str,
    runbook: str,
    priority: str,
    status: bool,
    used: int,
    vendor: str,
    RecomendationSources: str,
    RecomendationsCategory: str,
    userCreated: int,
    userApproved: int,
) -> recommendations:
    """This function adds recommendations.

    :param externalID: externalID
    :type externalID: str
    :param resource: resource
    :type resource: str
    :param shortDescription: shortDescription
    :type shortDescription: str
    :param LongDescription: LongDescription
    :type LongDescription: str
    :param command: command
    :type command: str
    :param runbook: runbook
    :type runbook: str
    :param priority: priority
    :type priority: str
    :param status: status
    :type status: bool
    :param used: used
    :type used: int
    :param vendor: vendor
    :type vendor: str
    :param RecomendationSources: RecomendationSources
    :type RecomendationSources::`~psym.graphql.enum.RecomendationSources
    :param RecomendationsCategory: RecomendationsCategory
    :type RecomendationsCategory: `~psym.graphql.enum.RecomendationsCategory
    :param userCreated: userCreated
    :type userCreated:`~psym.graphql.enum.user`
    :param userApproved: userApproved
    :type userApproved:`~psym.graphql.enum.user`

    :return: recommendations object
    :rtype: :class:`~psym.common.data_class.recommendations`


    **Example 1**

    .. code-block:: python

        recommendations_sources = client.add_recommendations_sources(
            name="recommendations_sources",
        )
        recommendations_category = client.add_recommendations_category(
            name="recommendations_category",
        )
        vendor = client.add_vendor(
            name="vendor",
        )

        user_created=client.get_user_by_email(email="user@nttdata.com")

        new_recommendations = client.add_recommendations(
            externalID="new_externalID",
            resource="resoruce",
            alarmType="alarmType",
            LongDescription="LongDescription",
            shortDescription="shortDescription",
            command="command",
            runbook="runbook",
            priority=2,
            status=False,
            used=2,
            vendor=vendor.id,
            RecomendationSources=recommendation_sources.id,
            RecomendationsCategory=recommendation_category.id,
            userCreated=user_created.id,
            userApproved=user_created.id
        )
        print(new_recommendations)
    """
    recommendations_input = AddRecommendationsInput(
    externalID=externalID,
    resource=resource,
    alarmType=alarmType,
    shortDescription=shortDescription,
    longDescription=LongDescription,
    priority=priority,
    status=status,
    recommendationsSources=RecomendationSources,
    recommendationsCategory=RecomendationsCategory,
    userCreate=userCreated,
    vendor=vendor,
    command=command,
    runbook=runbook,
    used=used,
    userApprobed=userApproved,
    )
    result = addRecommendations.execute(client, input=recommendations_input)
    return recommendations(
    id=result.id,
    externalID=result.externalID,
    resource=result.resource,
    alarmType=result.alarmType,
    shortDescription=result.shortDescription,
    LongDescription=result.longDescription,
    command=result.command,
    runbook=result.runbook,
    priority=result.priority,
    status=result.status,
    used=result.used,
    vendor=result.vendor,
    RecomendationSources=result.recommendationsSources,
    RecomendationsCategory=result.recommendationsCategory,
    userCreated=result.userCreate,
    userApproved=result.userApprobed
    )

def edit_recommendations(
    client: SymphonyClient,
    recommendations:recommendations,
    new_externalID:str=None,
    resource:str=None,
    alarmType:str=None,
    shortDescription:str=None,
    LongDescription:str=None,
    command:str=None,
    runbook:str=None,
    priority:int=None,
    status:bool=None,
    used:int=None,
    vendor:str=None,
    RecomendationSources:str=None,
    RecomendationsCategory:str=None,
    userApproved:str=None
) ->None:
    """This function edits recommendations.

    :param recommendations: recommendations entity
    :type name: str
    :param new_name: recommendations name
    :type name: str

    :return: none object
    :rtype: :class:`~psym.common.data_class.recommendations`

    **Example 1**

    .. code-block:: python

        recommendations_edited = client.edit_recommendations(
            recommendations=recommendations 
            new_externalID=new_externalID,
            resource="resoruce_2",
            alarmType="alarmType_2",
            LongDescription="LongDescription_2",
            shortDescription="shortDescription_2",
            command="command_2",
            runbook="runbook_2",
            priority=2,
            status=False,
            used=2,
            vendor=vendor.id,
            RecomendationSources=recommendation_sources.id,
            RecomendationsCategory=recommendation_category.id,
            userCreated=user_created.id,
            userApproved=user_created.id
            
            )
    """
    params: Dict[str, Any] = {}
    if new_externalID is not None:
        params.update({"_name_": new_externalID})
    if new_externalID is not None:
        editRecommendations.execute(client, input=EditRecommendationsInput(
            id=recommendations.id,
            externalID=new_externalID,
            resource=resource,
            alarmType=alarmType,
            shortDescription=shortDescription,
            longDescription=LongDescription,
            priority=priority,
            status=status,
            recommendationsSources=RecomendationSources,
            recommendationsCategory=RecomendationsCategory,
            vendor=vendor,
            command=command,
            runbook=runbook,
            used=used,
            userApprobed=userApproved))

def get_recommendations(client: SymphonyClient) -> Iterator[recommendations]:
    """Get the list of Recommendations
    :raises:
        FailedOperationException: Internal symphony error
    :return: Users Iterator
    :rtype: Iterator[ :class:`~psym.common.data_class.recommendations` ]
    **Example**
    .. code-block:: python
        recommendations_ = recommendations.get_recommendations()
        for recommendation in recommendations:
            print(recommendations.name)
    """
    result = Recommendations.execute(client)
    if result is None:
        return
    for edge in result.edges:
        node = edge.node
        if node is not None:
            yield recommendations(
                id=node.id,
                externalID=node.externalID,
                resource=node.resource,
                alarmType=node.alarmType,
                shortDescription=node.shortDescription,
                LongDescription=node.longDescription,
                command=node.command,
                runbook=node.resource,
                priority=node.priority,
                status=node.status,
                used=node.used,
                vendor=node.vendor,
                RecomendationSources=node.recommendationsSources,
                RecomendationsCategory=node.recommendationsCategory,
                userCreated=node.userCreate,
                userApproved=node.userApprobed

            )

def remove_recommendations(client: SymphonyClient, id: str) -> None:
    """This function delete recommendations.

    :param name: recommendations name
    :type name: :class:`~psym.common.data_class.recommendations`
    :rtype: None

    **Example**

    .. code-block:: python

        client.remove_recommendations(id=123456789)
    """	
    removeRecommendations.execute(client, id=id)
